/* libsegl - Sebas Extended GL Library
 *           Collection of Opengl/3D-Math helpers
 *
 *      Copyright (c) 2008 by Sebastian Lohff, seba@seba-geek.de
 *      http://www.seba-geek.de
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifndef __GLSDLSCREEN_H
#define __GLSDLSCREEN_H

#include <iostream>
#include <SDL.h>
#include <SDL_opengl.h>
#include "gltexture.h"

namespace segl {

class GLSDLScreen {
	private:
		SDL_Surface *screen;
		
		int width, height, bpp;
		bool opengl;
		bool glsetup;
		bool resizable;
		bool fullscreen;
		
		
		// OpenGL related
		float znear, zfar;
		void (*extraglparam)();
		int getFlags();
	public:
		GLSDLScreen();
		
		void enableOpenGL(bool);
		void enableResizable(bool);
		void enableFullscreen(bool);
		void setVideoMode(int _width, int _height, int _bpp);
		static void getVideoRes(std::string vidstr, int *w, int *h);
		
		void setGLNearFar(float _znear, float _zfar);
		void enableGLSetup(bool);
		void setExtraGLParamFunc(void (*extrafunc)());
		
		bool isOK();
		bool apply();
		
		int getWidth();
		int getHeight();
		
		void clearScreen();
		void renderScreen();
};

} // namespace segl

#endif
