/* libsegl - Sebas Extended GL Library
 *           Collection of Opengl/3D-Math helpers
 *
 *      Copyright (c) 2008 by Sebastian Lohff, seba@seba-geek.de
 *      http://www.seba-geek.de
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifndef __MATRIX_H
#define __MATRIX_H

#include <iostream>
#include <iomanip>

#include "punkt3d.h"
#include "emath.h"

namespace segl {

class Matrix {
	protected:
		float **c;
		int m, n;
	
		void allocate();
		void copyFromSource(const float**);
	public:
		Matrix(int, int);
		Matrix(int, int, const float**);
		Matrix(Punkt3D);
		Matrix(const Matrix&);
		~Matrix();
		
		const float **getMatrix() const;
		bool set(float, int, int);
		float get(int, int);
		int getM() const;
		int getN() const;
		
		Matrix operator*(const Matrix&);
		Matrix& operator=(const Matrix&);
		
		Matrix operator+(const float&);
		Matrix operator-(const float&);
		Matrix operator*(const float&);
		Matrix operator/(const float&);

		void print(std::string="") const;

};

} // namespace segl

#endif
