/* libsegl - Sebas Extended GL Library
 *           Collection of Opengl/3D-Math helpers
 *
 *      Copyright (c) 2008 by Sebastian Lohff, seba@seba-geek.de
 *      http://www.seba-geek.de
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifndef __PUNKT3D_H
#define __PUNKT3D_H

#include <iostream>
#include <cmath>
#include "emath.h"

namespace segl {

class Punkt2D {
	public:
		Punkt2D();
		Punkt2D(float, float);
		
		float x, y;
		
		void set(float, float);
		void print(std::string="");
		float abs();
		
		void normalize();
		Punkt2D getNormalized() const;
		
		// Operatoren
		Punkt2D operator+(const Punkt2D&);
		Punkt2D operator-(const Punkt2D&);
		Punkt2D& operator+=(const Punkt2D&);
		Punkt2D& operator-=(const Punkt2D&);
		
		Punkt2D operator+(const float&);
		Punkt2D operator-(const float&);
		Punkt2D operator*(const float&);
		Punkt2D operator/(const float&);
		Punkt2D& operator+=(const float&);
		Punkt2D& operator-=(const float&);
		Punkt2D& operator*=(const float&);
		Punkt2D& operator/=(const float&);
		
		float operator*(const Punkt2D&);
		
		Punkt2D operator-();
		
		bool operator==(const Punkt2D&);
		bool operator!=(const Punkt2D&);
		
		friend Punkt2D operator+(const float&, const Punkt2D&);
		friend Punkt2D operator-(const float&, const Punkt2D&);
		friend Punkt2D operator*(const float&, const Punkt2D&);
		friend Punkt2D operator/(const float&, const Punkt2D&);
		
};


} // namespace segl

void glTexCoord2f(segl::Punkt2D);
float abs(segl::Punkt2D);

// Fixed Headers
void glTexCoordP2D(segl::Punkt2D p);



#endif
